#include <stdio.h>
#include <stdlib.h>
#include <netinet/ip.h>     /* struct iphdr */
#include <unistd.h>         /* read */
#include <sys/socket.h>
#include <string.h>         /* strerror */
#include <argp.h>
#include <stdbool.h>

#include "ospf.h"
#include "chksum.h"
#include "msgtype.h"
#include "authtype.h"
#include "print.h"
#include "init.h"
#include "getaddr.h"
#include "getiface.h"
#include "verbose.h"

/* http://www.tenouk.com/Module43a.html */
#define PCKT_LEN 8192

bool quiet = false;
bool sendh = true;
int verbose = 0;

/**************************** ARGP ****************************/
static int parse_opt (int key, char * arg, struct argp_state * state)
{
  switch (key)
    {
    case 'v': verbose = 1; break;
    case 'q': quiet = true; break;
    case 's': sendh = false; break;
    }
  return 0;
}

struct argp_option options[] =
  {
    { 0, 'v', 0, 0, "explain what is being done"},
    { 0, 'q', 0, 0, "suppress packet header information"},
    { 0, 's', 0, 0, "simulate (don't actually send the hello"},
    { 0 }
  };
/**************************** ARGP ****************************/

int main (int argc, char ** argv)
{
  struct iphdr * ip;
  struct OSPFheader * header;
  struct OSPFhello * hello;
  char * iface;
  
  uint8_t buffer[PCKT_LEN];
  int sd;
  struct sockaddr_in sin;
  const int on = 1;

  uint32_t address, netmask;
  
  /**************************** ARGP ****************************/
  struct argp argp = { options, parse_opt };
  argp_parse (&argp, argc, argv, 0, 0, 0);
  /**************************** ARGP ****************************/
  
  memset(buffer, 0, PCKT_LEN);

  ip = (struct iphdr *) buffer;
  header = (struct OSPFheader *) (buffer + sizeof(struct iphdr));
  hello = (struct OSPFhello *) (buffer + sizeof(struct iphdr) +
		 	        sizeof(struct OSPFheader));

  /************** FIX THIS *******************
  if (argc > 1)
    iface = argv[1];
  else
    iface = getDefaultInterface();
   ************** FIX THIS *******************/

  iface = getDefaultInterface();

  address = getAddress(iface);
  netmask = getNetmask(iface);

  initIP(ip, address);
  initOSPFHeader(header, address);
  initOSPFHello(hello, address, netmask);
  
  ip->tot_len = sizeof(struct iphdr) + sizeof(struct OSPFheader) +
    sizeof(struct OSPFhello); /* single-byte field, so no htons */

  header->length = htons(sizeof(struct OSPFheader) + sizeof(struct OSPFhello));

  ip->check = 0;
  ip->check = chksum(ip, ip->ihl * 4); // interesting fact: do not change byte ordering

  header->chksum = 0;
  header->chksum = chksum(header, ntohs(header->length));

  if (!quiet)
    {
      printIP(ip);
      printOSPFHeader(header);
      printOSPFHello(hello);
    }
  
    if (sendh)
      {
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = ip->daddr;

  sd = socket(PF_INET, SOCK_RAW, IPPROTO_RAW);
  if (sd < 0)
    {
      perror("socket() error");
      exit(EXIT_FAILURE);
    }

  if (setsockopt(sd, IPPROTO_IP, IP_HDRINCL, &on, sizeof(on)) < 0)
    {
      perror("setsockopt() error");
      exit(EXIT_FAILURE);
    }

  if (sendto(sd, buffer, ip->tot_len, 0, (struct sockaddr *)&sin,
	     sizeof(struct sockaddr)) < 0)
    {
      perror("sendto() failed");
      exit(EXIT_FAILURE);
    }
  
  close(sd);
    }
  return(EXIT_SUCCESS);
}
