#include <stdio.h>
#include <netinet/in.h>     
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <netinet/ip.h>     /* struct iphdr */
#include <netdb.h>          /* /etc/protocols */

#include "print.h"
#include "authtype.h"
#include "msgtype.h"
#include "options.h"

void printOSPFHeader(struct OSPFheader * header)
{
  char rid [INET_ADDRSTRLEN], areaid [INET_ADDRSTRLEN];
  inet_ntop (AF_INET, &(header->rid), rid, INET_ADDRSTRLEN);
  inet_ntop (AF_INET, &(header->areaid), areaid, INET_ADDRSTRLEN);
  
  printf("Open Shortest Path First\n");
  printf("\tVersion: %d\n", header->version);
  printf("\tMessage Type: %s (%d)\n",
	 getOSPFMessageType(header->msgtype), header->msgtype);
  printf("\tPacket Length: %d\n", ntohs(header->length));
  printf("\tSource OSPF Router: %s (%s)\n", rid, rid);
  printf("\tArea ID: %s (%s)", areaid, areaid);
  if (header->areaid == 0)
    printf(" (Backbone)");
  printf("\n");
  printf("\tChecksum: %#04x\n", ntohs(header->chksum));
  printf("\tAuth Type: %s (%d)\n", getOSPFAuthType(ntohs(header->autype)),
	 ntohs(header->autype));
  printf("\tAuth Data");
  if (header->authen == 0)
    printf(" (none)");
  printf(": %016llx", (long long unsigned int)header->authen);
  if (header->authen != 0)
    printf("(warning: byte order not corrected!)");
  printf("\n");
}

void printIP (struct iphdr * ip)
{
  char saddr [INET_ADDRSTRLEN], daddr [INET_ADDRSTRLEN];
  inet_ntop (AF_INET, &(ip->saddr), saddr, INET_ADDRSTRLEN);
  inet_ntop (AF_INET, &(ip->daddr), daddr, INET_ADDRSTRLEN);

  printf("Internet Protocol Version 4\n");
  printf("\tVersion: %d\n", ip->version);
  printf("\tHeader Length: %d\n", ip->ihl);
  printf("\tDifferentiated Services Field: %#02x\n", ip->tos);
  printf("\tTotal Length: %d\n", ip->tot_len);
  printf("\tIdentification: %#04x\n", ntohs(ip->id));
  printf("\tFlags: %#02x\n", ip->frag_off);
  printf("\tFragment Offset: %d\n", ip->frag_off);
  printf("\tTime to live: %d\n", ip->ttl);
  printf("\tProtocol: %s (%d)\n", getprotobynumber(ip->protocol)->p_aliases[0],
	 ip->protocol);
  printf("\tHeader checksum: %#04x\n", ip->check);
  printf("\tSource: %s\n", saddr);
  printf("\tDestination: %s\n", daddr);
}

void printOSPFHello(struct OSPFhello * hello)
{
  char mask [INET_ADDRSTRLEN], dr [INET_ADDRSTRLEN], bdr [INET_ADDRSTRLEN];
  inet_ntop (AF_INET, &(hello->netmask), mask, INET_ADDRSTRLEN);
  inet_ntop (AF_INET, &(hello->dr), dr, INET_ADDRSTRLEN);
  inet_ntop (AF_INET, &(hello->bdr), bdr, INET_ADDRSTRLEN);

  printf("OSPF Hello Packet\n");
  printf("\tNetwork Mask: %s\n", mask);
  printf("\tHello Interval [sec]: %d\n", ntohs(hello->hellointerval));
  printf("\tOptions: 0x%02x\n", hello->options);
  printOSPFHelloOptions(hello->options);
  printf("\tRouter Priority: %d\n", hello->priority);
  printf("\tRouter Dead Interval [sec]: %d\n", ntohl(hello->deadint));
  printf("\tDesignated Router: %s\n", dr);
  printf("\tBackup Designated Router: %s\n", bdr);
}

void printOSPFHelloOptions(uint8_t options)
{
  uint8_t dn, o, dc, l, np, mc, e, mt;

  dn = (options & OSPF_OPTION_DN) >> 7;
  o  = (options & OSPF_OPTION_O ) >> 6;
  dc = (options & OSPF_OPTION_DC) >> 5;
  l  = (options & OSPF_OPTION_L ) >> 4;
  np = (options & OSPF_OPTION_NP) >> 3;
  mc = (options & OSPF_OPTION_MC) >> 2;
  e  = (options & OSPF_OPTION_E ) >> 1;
  mt = options & OSPF_OPTION_MT;

  
  printf("\t  %d... .... = DN: %s\n", dn, dn ? "Set" : "Not set");
  printf("\t  .%d.. .... = O: %s\n", o, o ? "Set" : "Not set");
  printf("\t  ..%d. .... = DC: Demand Circuits are %ssupported\n", dc, dc ? "" : "NOT ");
  printf("\t  ...%d .... = L: This packet does %scontain LLS data block\n", l, l ? "" : "NOT ");
  printf("\t  .... %d... = NP: NSSA is %ssupported\n", np, np ? "" : "NOT ");
  printf("\t  .... .%d.. = MC: %sMulticast Capable\n", mc, mc ? "" : "NOT ");
  printf("\t  .... ..%d. = E: %sExternal Routing Capability\n", e, e ? "" : "NO ");
  printf("\t  .... ...%d = MT: %sMulti-Topology Routing\n", mt, mt ? "" : "NO ");
}
